// based on the esp-now tutorial found here:
// https://randomnerdtutorials.com/esp-now-esp32-arduino-ide/

#include <esp_now.h>
#include <WiFi.h>

#include "USB.h"
#include "USBHIDKeyboard.h"
USBHIDKeyboard Keyboard;

const int buttonPin = 41;         // input pin for pushbutton
int previousButtonState = HIGH;   // for checking the state of a pushButton
int counter = 0;                  // button push counter
const int led = 21;               // led 

// Structure example to receive data
// Must match the sender structure
typedef struct struct_message {
  int id;
  int key;
}struct_message;

// Create a struct_message called myData
struct_message myData;

// Create a structure to hold the readings from each board
struct_message board1;
struct_message board2;
struct_message board3;

// Create an array with all the structures
struct_message boardsStruct[3] = {board1, board2, board3};

// callback function that will be executed when data is received
void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  char macStr[18];

  digitalWrite(led, HIGH);
  
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&myData, incomingData, sizeof(myData));
  Serial.printf("Board ID %u: %u bytes\n", myData.id, len);
  // Update the structures with the new incoming data
  boardsStruct[myData.id-1].key = myData.key;
  Serial.printf("Key value: %d \n", boardsStruct[myData.id-1].key);
  Serial.println();

  if (boardsStruct[myData.id-1].key == 5) {
    Keyboard.press(KEY_LEFT_GUI);
    Keyboard.press('N');
    delay(250);
    Keyboard.releaseAll();
    delay(100);
    Keyboard.print("hackaday.com");
    Keyboard.press(KEY_RETURN);
    delay(100);
    Keyboard.releaseAll();
  } else { 
    Keyboard.print("NA");
  }
  
  digitalWrite(led, LOW);
} 

void setup() {
  pinMode(led, OUTPUT);
  // make the pushButton pin an input:
  pinMode(buttonPin, INPUT_PULLUP);
  // initialize control over the keyboard:
  Keyboard.begin();
  USB.begin();

  //Set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  //Init ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  esp_now_register_recv_cb(OnDataRecv);
}

void loop() {
  // read the pushbutton:
  int buttonState = digitalRead(buttonPin);
  // if the button state has changed,
  if ((buttonState != previousButtonState)
      // and it's currently pressed:
      && (buttonState == LOW)) {
    // increment the button counter
    counter++;
    // type out a message
    Keyboard.println("test");
  }
  // save the current button state for comparison next time:
  previousButtonState = buttonState;
}
